// Wave File Class
// Adapted for GNU/Linux Will 18, January 1998
// @Ximon Eighteen, March 1997
// Original code written by Yung-Hsiang Lee

#ifndef WAVECLASS_H
#define WAVECLASS_H


// ## ( Compiler includes ) ##
#include <stdlib.h>
#include <stdio.h>
/* needed for typedef Boolean */
#include <X11/Intrinsic.h>

//  ## (  Custom Constants  ) ##
const short PCM_WAV = 1;


//  ## (  Begin: Wav header structures  ) ##
//  ## (  Author: Yung-Hsiang Lee  		) ##
//  ## (  Notes: Lifted from the PC-GPE ) ##
//  ## (  PC Games Programmers' Encycl. ) ##


// File header
typedef struct              	// RIFF formatting information
{                             // RIFF file header
   char riff[4];					// Contains 'R' 'I' 'F' 'F'
   long filesize __attribute__ ((packed));
   char rifftype[4] __attribute__ ((packed));          // Contains 'W' 'A' 'V' 'E'
} RiffHeader;

typedef struct                // RIFF formatting information
{										// RIFF chunk/block information
   char chunk_id[4] __attribute__ ((packed));
   long chunksize __attribute__ ((packed));
} Chunk;

typedef struct
{
   short wFormatTag __attribute__ ((packed));				// 1 = PCM (8-bit uncompressed)
   short nChannels __attribute__ ((packed));				// 1 = Mono, 2 = Stereo
   long nSamplesPerSec __attribute__ ((packed));			// Playback frequency
   long nAvgBytesPerSec __attribute__ ((packed));		// Bytes per second that should be transferred (A)
   short nBlockAlign __attribute__ ((packed));			// Size of data block to process at a time (B)  
} WAVEFORMAT;

//  ## (  Begin: Footnotes  ) ##

//	(A) - Indicates the average number of bytes per second that data
//			should be transferred at :
//				nAvgBytesPerSec = nChannels * nSamplesPerSec *
//										(nBitsPerSample / 8)
   	
//	(B) - Indicates the block alignment of the data in the data chunk.
//			Software needs to process a multiple of nBlockAlign at a time.
//				nBlockAlign = nChannels * (nBitsPerSample / 8)

//  ## (  End: Footnotes  ) ##


typedef struct
{
   WAVEFORMAT wf;
   short wBitsPerSample __attribute__ ((packed));	// 16-bit, 8-bit, etc.
// default is to pad to long (4 bytes)
} PCMWAVEFORMAT;

typedef struct
{                                                           	 
	RiffHeader header;		// File header							 
	Chunk ch_format;		// Format chunk id: 'F' 'M' 'T'   
	PCMWAVEFORMAT format;	// 	|--> Format chunk data	    
	Chunk ch_data;			// Data chunk id: 'D' 'A' 'T' 'A' 
	// Data follows here.	// 	|--> Data chunk data			 
} WaveFile;

//  ## (  End: Wav header structures  ) ##


//  ## (  Begin: Wav file handling class  ) ##
//  ## (  Author: Ximon Eighteen  ) ##

class Wave
{
	private:
		WaveFile waveHeader;
		FILE *wavFile;
		char *pName;      			
		char *pData;
		unsigned int _waveNumber;
		int _headerSize;

		void Init(void);
		void Set(short,long,short);
		Boolean IsWavFile(void);
		Boolean IsAltWavFile(void);
         
	protected:
		static Wave *pFirst;
		Wave *pNext;
		static unsigned int _waveCount;

	public:
		Wave();
		~Wave();

		long filePos;

		Boolean AllocateDataSpace(long bytes);
		void Free(void);

		Boolean LoadWav(String fileName);
		Boolean LoadWav(String fileName, short wavType);
		Boolean LoadHeader(String fileName);
		Boolean LoadHeader(String fileName, short wavType);
		Boolean SaveWav(String fileName);
		Boolean SaveHeader(String fileName);
		Boolean ChangeHeader(String fileName);
		Boolean AppendBlock(String fileName,short channels,long sampleRate,
							short dataSize,long numSamples);
		void RewindToStart(void);
		int HeaderSize(void) { return sizeof(waveHeader); };
		long LoadBlock(long filePos, long blockLength);
		Boolean SaveBlock(String fileName, long filePos, long blockLength, void *data);

		Boolean MajMin(String filename);

		WaveFile *GetHeader(void);

  		void waveNum ( unsigned int num );	// should only be called by external delete to keep numbers in sync
		unsigned int waveNum(void);
		char *name(void);				// address of name space
		char *data(void);				// address of data space
		short waveType(void);			// 1 = PCM
		short numChannels(void);		// 1 = MONO, 2 = STEREO
		short bitsPerSample(void);		// 8-bit, 16-bit etc.
		long sampleRate(void);			// 44100, ETC.
		short blockAlign(void);
		long dataLength(void);			// in bytes
		char *dataChunkId(void);
		long formatLength(void);			// in bytes
		char *formatChunkId(void);
		char *riffHeader(void);
		long fileSize(void);
		char *riffType(void);
		long totalSamples(void);      
		void FillHeader(short channels,long sampleRate,short dataSize,
						long totalSamples);

		static Wave *first() { return pFirst; }

		Wave *next() { return pNext; }

		static int waveCount() { return _waveCount; }
};
 

//  ## (  End: Wav file handling class  ) ##


#endif


